"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = __importDefault(require("typescript"));
const vitest_1 = require("vitest");
const utils_test_1 = require("./utils-test");
const imports_1 = require("./imports");
const utils_1 = require("./utils");
const visit_var_def_1 = require("./visit-var-def");
const visit_expression_1 = require("./visit-expression");
(0, vitest_1.describe)("All types implement Resource", async () => {
    const sourceCode = `
import { Resource } from "@plutolang/base";

class DirectCls implements Resource {}
interface DirectIntf extends Resource {}
interaface DirectCls extends Resource {} // same name with DirectCls

class IndirectCls implements DirectIntf {}
interface IndirectIntf extends DirectIntf {}
interface IndirectCls extends DirectIntf {} // same name with IndirectCls
`;
    const { sourceFile, checker } = (0, utils_test_1.genAnalyzerForInline)(sourceCode);
    typescript_1.default.forEachChild(sourceFile, (node) => {
        if (typescript_1.default.isInterfaceDeclaration(node) || typescript_1.default.isClassDeclaration(node)) {
            (0, vitest_1.test)(node.getText(), async () => {
                (0, vitest_1.expect)((0, utils_1.isResourceType)(node, checker)).toBe(true);
            });
        }
    });
});
(0, vitest_1.describe)("All types don't implement Resource", async () => {
    const sourceCode = `
import { Resource } from "@plutolang/other";

class DirectCls implements Resource {}
interface DirectIntf extends Resource {}
interaface DirectCls extends Resource {} // same name with DirectCls

class IndirectCls implements DirectIntf {}
interface IndirectIntf extends DirectIntf {}
interface IndirectCls extends DirectIntf {} // same name with IndirectCls
`;
    const { sourceFile, checker } = (0, utils_test_1.genAnalyzerForInline)(sourceCode);
    typescript_1.default.forEachChild(sourceFile, (node) => {
        if (typescript_1.default.isInterfaceDeclaration(node) || typescript_1.default.isClassDeclaration(node)) {
            (0, vitest_1.test)(node.getText(), async () => {
                (0, vitest_1.expect)((0, utils_1.isResourceType)(node, checker)).toBe(false);
            });
        }
    });
});
(0, vitest_1.describe)("Valid resource variable statements", async () => {
    const sourceCode = `
import { Router } from "@plutolang/pluto";
import * as pluto from "@plutolang/pluto";

const obj1 = new Router("obj1");
const obj2 = new Router("obj2"), obj3 = new pluto.Router("obj3"), obj4;
`;
    const { sourceFile, checker } = (0, utils_test_1.genAnalyzerForFile)(sourceCode);
    typescript_1.default.forEachChild(sourceFile, (node) => {
        if (typescript_1.default.isVariableStatement(node)) {
            (0, vitest_1.test)("check if there are resource variables: " + node.getText(), async () => {
                const resNum = node.getText().match(/new/g)?.length ?? 0;
                const resVarInfos = (0, visit_var_def_1.visitVariableStatement)(node, checker);
                (0, vitest_1.expect)(resVarInfos).toHaveLength(resNum);
                const nameImportElem = {
                    type: imports_1.ImportType.Named,
                    name: "Router",
                    package: "@plutolang/pluto",
                };
                const nsImportElem = {
                    type: imports_1.ImportType.Namespace,
                    name: "pluto",
                    package: "@plutolang/pluto",
                };
                for (const resVarInfo of resVarInfos) {
                    switch (resVarInfo.varName) {
                        case "obj1":
                            (0, vitest_1.expect)(resVarInfo.resourceConstructInfo.constructExpression).toEqual("Router");
                            (0, vitest_1.expect)(resVarInfo.resourceConstructInfo.importElements[0]).toEqual(nameImportElem);
                            break;
                        case "obj2":
                            (0, vitest_1.expect)(resVarInfo.resourceConstructInfo.constructExpression).toEqual("Router");
                            (0, vitest_1.expect)(resVarInfo.resourceConstructInfo.importElements[0]).toEqual(nameImportElem);
                            break;
                        case "obj3":
                            (0, vitest_1.expect)(resVarInfo.resourceConstructInfo.constructExpression).toEqual("pluto.Router");
                            (0, vitest_1.expect)(resVarInfo.resourceConstructInfo.importElements[0]).toEqual(nsImportElem);
                            break;
                    }
                }
            });
        }
    });
    (0, utils_test_1.rmSourceFile)(sourceFile);
});
(0, vitest_1.describe)("Valid expression", async () => {
    const sourceCode = `
import { Router, HttpRequest, HttpResponse } from "@plutolang/pluto";
import * as pluto from "@plutolang/pluto";

let obj1;
obj1 = new Router("obj1");
obj1 = 1;

let obj2, obj3, obj4;
obj2 = new Router("obj2"), obj3 = new Router("obj3"), obj4 = new Router("obj4");

obj3.get("/", async (req: HttpRequest) => Promise<HttpResponse> {
  return {
    statusCode: 200,
    body: "hello"
  };
});
`;
    const { sourceFile, checker } = (0, utils_test_1.genAnalyzerForFile)(sourceCode);
    typescript_1.default.forEachChild(sourceFile, (node) => {
        if (typescript_1.default.isExpressionStatement(node)) {
            const childNode = node.expression;
            if (typescript_1.default.isBinaryExpression(childNode)) {
                (0, vitest_1.test)("check binary expression: " + childNode.getText(), async () => {
                    const resNum = node.getText().match(/new/g)?.length ?? 0;
                    const resVarInfos = (0, visit_expression_1.visitBinaryExpression)(childNode, checker);
                    (0, vitest_1.expect)(resVarInfos).toHaveLength(resNum);
                });
            }
            else if (typescript_1.default.isCallExpression(childNode)) {
                (0, vitest_1.test)("check call expression: " + childNode.getText().split("\n")[0], async () => {
                    const union = (0, visit_expression_1.visitCallExpression)(childNode, checker);
                    (0, vitest_1.expect)(union?.resourceVarInfos).toHaveLength(1);
                    (0, vitest_1.expect)(union?.resourceRelatInfos).toHaveLength(1);
                });
            }
        }
    });
    (0, utils_test_1.rmSourceFile)(sourceFile);
});
(0, vitest_1.describe)("Invalid call expression", async () => {
    const sourceCode = `
import { Router, HttpRequest, HttpResponse } from "@plutolang/pluto";
import * as pluto from "@plutolang/pluto";

let obj2 = new Router("obj2");

const getFn = obj2.get;
getFn("/", async (req: HttpRequest) => Promise<HttpResponse> {
  return {
    statusCode: 200,
    body: "hello"
  };
});

const getFn2 = getFn;
getFn2("/", async function hello (req: HttpRequest) => Promise<HttpResponse> {
  return {
    statusCode: 200,
    body: "hello"
  };
});

class Cls {
  public add(a: number, b: number): number;
  public add(a: number): number;

  public add(a: number, b: number = 1): number {
    return a + b;
  }
}

const cls = new Cls();
cls.add(1);
`;
    const { sourceFile, checker } = (0, utils_test_1.genAnalyzerForFile)(sourceCode);
    typescript_1.default.forEachChild(sourceFile, (node) => {
        if (typescript_1.default.isExpressionStatement(node)) {
            const childNode = node.expression;
            if (typescript_1.default.isCallExpression(childNode)) {
                (0, vitest_1.test)("check call expression: " + childNode.getText().split("\n")[0], async () => {
                    (0, vitest_1.expect)(() => (0, visit_expression_1.visitCallExpression)(childNode, checker)).toThrow();
                });
            }
        }
    });
    (0, utils_test_1.rmSourceFile)(sourceFile);
});
(0, vitest_1.describe)("Valid constants accessing", async () => {
    const sourceCode = `
import { Router, HttpRequest, HttpResponse, Queue } from "@plutolang/pluto";
const router = new Router("router");

const constNum = 10;
const constStr = "Hello World!";

const aliasNum = constNum;

const queue = new Queue("queue");
const aliasQueue = queue;

router.get("/store", async (req: HttpRequest): Promise<HttpResponse> => {
  console.log(constNum);
  const anotherNum = constNum + 1;
  const anotherStr = constStr;
  
  const accessAliasNum = aliasNum;

  await queue.push("foo");
  await aliasQueue.push("bar");
  
  return {
    statusCode: 200,
    body: \`Fetch access message.\`,
  };
});
`;
    const { sourceFile, checker } = (0, utils_test_1.genAnalyzerForFile)(sourceCode);
    typescript_1.default.forEachChild(sourceFile, (node) => {
        if (typescript_1.default.isExpressionStatement(node)) {
            const childNode = node.expression;
            if (!typescript_1.default.isCallExpression(childNode)) {
                return;
            }
            (0, vitest_1.test)("check call expression: " + childNode.getText().split("\n")[0], async () => {
                const union = (0, visit_expression_1.visitCallExpression)(childNode, checker);
                (0, vitest_1.expect)(union?.resourceVarInfos).toHaveLength(1);
                (0, vitest_1.expect)(union?.resourceVarInfos[0].resourceConstructInfo.locations).toHaveLength(4);
            });
        }
    });
    (0, utils_test_1.rmSourceFile)(sourceFile);
});
(0, vitest_1.describe)("Invalid constants accessing", async () => {
    const sourceCode = `
import { Router, HttpRequest, HttpResponse, Queue } from "@plutolang/pluto";
const router = new Router("router");

const constArr = [1, 2, 3];

router.get("/store", async (req: HttpRequest): Promise<HttpResponse> => {
  constArr.push(3); // not ensure the value can be updated.
  return {
    statusCode: 200,
    body: \`Fetch access message.\`,
  };
});
`;
    const { sourceFile, checker } = (0, utils_test_1.genAnalyzerForFile)(sourceCode);
    typescript_1.default.forEachChild(sourceFile, (node) => {
        if (typescript_1.default.isExpressionStatement(node)) {
            const childNode = node.expression;
            if (!typescript_1.default.isCallExpression(childNode)) {
                return;
            }
            (0, vitest_1.test)("check call expression: " + childNode.getText().split("\n")[0], async () => {
                (0, vitest_1.expect)(() => (0, visit_expression_1.visitCallExpression)(childNode, checker)).toThrowError();
            });
        }
    });
    (0, utils_test_1.rmSourceFile)(sourceFile);
});
(0, vitest_1.describe)("Valid function calling", async () => {
    const sourceCode = `
import { Router, HttpRequest, HttpResponse, Queue } from "@plutolang/pluto";
const router = new Router("router");

const queue = new Queue("queue");

const constInFn = 10;
async function test(): Promise<string> {
  await queue.push("foo");
  console.log(constInFn);
  return "hello";
}

router.get("/store", async (req: HttpRequest): Promise<HttpResponse> => {
  const str = await test();
  return {
    statusCode: 200,
    body: \`Fetch access message. \${test2()}\`,
  };
});
`;
    const { sourceFile, checker } = (0, utils_test_1.genAnalyzerForFile)(sourceCode);
    typescript_1.default.forEachChild(sourceFile, (node) => {
        if (typescript_1.default.isExpressionStatement(node)) {
            const childNode = node.expression;
            if (!typescript_1.default.isCallExpression(childNode)) {
                return;
            }
            (0, vitest_1.test)("check call expression: " + childNode.getText().split("\n")[0], async () => {
                const union = (0, visit_expression_1.visitCallExpression)(childNode, checker);
                (0, vitest_1.expect)(union?.resourceVarInfos).toHaveLength(1); // lambda
                (0, vitest_1.expect)(union?.resourceRelatInfos).toHaveLength(2); // router->lambda; lambda->queue
                (0, vitest_1.expect)(union?.resourceVarInfos[0].resourceConstructInfo.locations).toHaveLength(3);
            });
        }
    });
    (0, utils_test_1.rmSourceFile)(sourceFile);
});
//# sourceMappingURL=deducer.test.js.map